﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {
  [StorableClass]
  [Item("AngularDistance", "The Angluar Distance as defined as a normalized distance measure dependent on the angle between two vectors.\nIt is designed for vectors with all positive coordinates")]
  public class AngularDistance : DistanceBase<IEnumerable<double>> {

    #region HLConstructors & Boilerplate
    [StorableConstructor]
    protected AngularDistance(bool deserializing) : base(deserializing) { }
    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() { }
    protected AngularDistance(AngularDistance original, Cloner cloner)
      : base(original, cloner) { }
    public AngularDistance() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AngularDistance(this, cloner);
    }
    #endregion

    #region statics
    public static double GetDistance(double[] point1, double[] point2) {
      if (point1.Length != point2.Length) throw new ArgumentException("Angluar distance not defined on vectors of different length");
      var innerProduct = point1.Zip(point2, (x, y) => x * y).Sum();
      return Math.Acos(innerProduct / (Norm(point1) * Norm(point2))) / Math.PI;
    }
    public static double GetDistance(IEnumerable<double> a, IEnumerable<double> b, double threshold) {
      return GetDistance(a.ToArray(), b.ToArray());     //no shortcut evaluation for Angular Distance
    }
    #endregion
    public override double Get(IEnumerable<double> a, IEnumerable<double> b) {
      return GetDistance(a.ToArray(), b.ToArray());
    }
    public override double Get(IEnumerable<double> a, IEnumerable<double> b, double threshold) {
      return GetDistance(a, b, threshold);
    }

    private static double Norm(IEnumerable<double> point) {
      return Math.Sqrt(point.Sum(d => d * d));
    }
  }
}
